@@ -31,7 +31,7 @@ class TorchNet(object):
             for m in self._models:
                 m.eval()
             self._gpulist = idlist
-
+        torch.set_grad_enabled(False)            
         return self
 
     def iscpu(self):
@@ -46,8 +46,7 @@ class TorchNet(object):
            * Note: Do not use DataParallel, this replicates the multi-gpu batch on device 0 and results in out of memory
         """
         assert len(t) <= self.batchsize(), "Invalid batch size"
-        t = t.pin_memory() if self.isgpu() else t
-        todevice = [b.to(d, non_blocking=True) for (b,d) in zip(t.split(self._batchsize) , self._devices)]  # async?
+        todevice = [b.pin_memory().to(d, non_blocking=True) for (b,d) in zip(t.split(self._batchsize) , self._devices)]  # async?
         fromdevice = [m(b) for (m,b) in zip(self._models, todevice)]   # async?
         return torch.cat([r.detach().cpu() for r in fromdevice], dim=0)
         
@@ -109,6 +108,7 @@ class Yolov5(TorchNet):
         self._device = None
         self._gpulist = gpu
         self.gpu(gpu, batchsize)
+        torch.set_grad_enabled(False)
         
     def __call__(self, imlist, conf=1E-3, iou=0.5, union=False, objects=None):
         """Run detection on an image list at specific mininum confidence and iou NMS
@@ -122,15 +122,14 @@ class Yolov5(TorchNet):
 
         imlist = tolist(imlist)
         imlistdets = []
-        t = torch.cat([im.clone(shallow=True).maxsquare().mindim(self._mindim).gain(1.0/255.0).torch() for im in imlist]).type(self._tensortype)  # triggers load
+        t = torch.cat([im.clone(shallow=True).maxsquare().mindim(self._mindim).gain(1.0/255.0).torch() for im in imlist])  # triggers load
 
         assert len(t) <= self.batchsize(), "Invalid batch size"
-        t = t.pin_memory() if self.isgpu() else t
-        todevice = [b.to(d, non_blocking=True) for (b,d) in zip(t.split(self._batchsize) , self._devices)]  # async?
+        todevice = [b.pin_memory().to(d, non_blocking=True) for (b,d) in zip(t.split(self._batchsize) , self._devices)]  # async?
         fromdevice = [m(b)[0] for (m,b) in zip(self._models, todevice)]     # detection!
         t_out = [torch.squeeze(t, dim=0) for d in fromdevice for t in torch.split(d, 1, 0)]   # unpack batch to list of detections per imag
         t_out = [torch.cat((t[:,0:5], torch.argmax(t[:,5:], dim=1, keepdim=True)), dim=1) for t in t_out]  # filter argmax on device 
-        t_out = [t[t[:,4]>conf].detach().cpu().numpy() for t in t_out]  # filter conf on device (this must be last)
+        t_out = [t[t[:,4]>conf].cpu().numpy() for t in t_out]  # filter conf on device (this must be last)
 
         k_valid_objects = set([self._cls2index[k] for k in objects.keys()]) if objects is not None else self._cls2index.values()        
         for (im, dets) in zip(imlist, t_out):
@@ -147,7 +146,7 @@ class Yolov5(TorchNet):
                                  
                 scale = max(im.shape()) / float(self._mindim)  # to undo
                 objectlist = [obj.rescale(scale) for obj in objectlist]
-                objectlist = [obj.category(objects[obj.category()]) if objects is not None else obj for obj in objectlist]
+                objectlist = [obj.category(objects[obj.category()]) if objects is not None else obj for obj in objectlist]  # convert to target class before NMS
             else:
                 objectlist = []
 
@@ -232,7 +231,7 @@ class ObjectDetector(Yolov5):
 
 class MultiscaleObjectDetector(ObjectDetector):  
     """Given a list of images, break each one into a set of overlapping tiles, and ObjectDetector() on each, then recombining detections"""
-    def __call__(self, imlist, conf=0.5, iou=0.5, maxarea=1.0, objects=None, overlapfrac=6, filterborder=True):  
+    def __call__(self, imlist, conf=0.5, iou=0.5, maxarea=1.0, objects=None, overlapfrac=6, filterborder=True, cover=0.7):  
         (f, n) = (super().__call__, self._mindim)
         assert isinstance(imlist, vipy.image.Image) or isinstance(imlist, list) and all([isinstance(im, vipy.image.Image) for im in imlist]), "invalid input"
         imlist = tolist(imlist)
@@ -240,11 +239,28 @@ class MultiscaleObjectDetector(ObjectDetector):
         
         (imlist_multiscale, imlist_multiscale_flat, n_coarse, n_fine) = ([], [], [], [])
         for im in imlist:
-            imcoarse = [im]            
-            imfine = (im.tile(n, n, overlaprows=im.height()-n, overlapcols=(3*n-im.width())//2) if (im.mindim()>n and im.mindim() == im.height()) else
-                      (im.tile(n, n, overlapcols=im.width()-n, overlaprows=(3*n-im.height())//2) if im.mindim()>n else []))  # 2x3 tile, assumes im.mindim() == (n+n/2)
-            if len(imfine) != 6:
-                print('WARNING: len(imtile) = %d' % len(imfine))  # Sanity check
+            imcoarse = [im]
+
+            # FIXME: generalize this parameterization
+            if overlapfrac == 6:
+                imfine = (im.tile(n, n, overlaprows=im.height()-n, overlapcols=(3*n-im.width())//2) if (im.mindim()>=n and im.mindim() == im.height()) else
+                          (im.tile(n, n, overlapcols=im.width()-n, overlaprows=(3*n-im.height())//2) if im.mindim()>=n else []))  # 2x3 tile, assumes im.mindim() == (n+n/2)
+                if len(imfine) != 6:
+                    print('WARNING: len(imtile) = %d for overlapfrac = %d' % (len(imfine), overlapfrac))  # Sanity check                    
+                    
+            elif overlapfrac == 2:
+                imfine = (im.tile(n, n, overlaprows=0, overlapcols=(2*n-im.width())//2) if (im.mindim()>=n and im.mindim() == im.height()) else
+                          (im.tile(n, n, overlapcols=0, overlaprows=(2*n-im.height())//2) if im.mindim()>=n else []))  # 1x2 tile, assumes im.mindim() == (n)
+                if len(imfine) != 2:
+                    print('WARNING: len(imtile) = %d for overlapfrac = %d' % (len(imfine), overlapfrac))  # Sanity check
+                    
+            elif overlapfrac == 0:
+                imfine = []
+                
+            else:
+                raise
+            # /FIXME
+            
             n_coarse.append(len(imcoarse))
             n_fine.append(len(imfine))
             imlist_multiscale.append(imcoarse+imfine)
@@ -258,14 +274,16 @@ class MultiscaleObjectDetector(ObjectDetector):
             imcoarsedet = im_multiscale[0].mindim(iml.mindim())
             imcoarsedet_imagebox = imcoarsedet.imagebox()
             if filterborder:
-                imfinedet = iml.untile( [im.nms(conf, iou).objectfilter(lambda o: (o.area()<=maxarea*im.area() and   # not too big relative to tile
-                                                                                   (o.isinterior(im.width(), im.height(), border=0.9) or  # not occluded by any tile boundary 
-                                                                                    o.clone().dilatepx(0.1*im.width()+1).cover(im.imagebox()) == o.clone().dilatepx(0.1*im.width()+1).set_origin(im.attributes['tile']['crop']).cover(imcoarsedet_imagebox))))  # or only occluded by image boundary
-                                         for im in im_multiscale[nc:]] )
+                imfinedet = [im.nms(conf, iou, cover=cover).objectfilter(lambda o: ((maxarea==1 or (o.area()<=maxarea*im.area())) and   # not too big relative to tile
+                                                                                    ((o.isinterior(im.width(), im.height(), border=0.9) or  # not occluded by any tile boundary 
+                                                                                      o.clone().dilatepx(0.1*im.width()+1).cover(im.imagebox()) == o.clone().dilatepx(0.1*im.width()+1).set_origin(im.attributes['tile']['crop']).cover(imcoarsedet_imagebox)))))  # or only occluded by image boundary
+                             for im in im_multiscale[nc:]]
+                imfinedet = [im.objectmap(lambda o: o.set_origin(im.attributes['tile']['crop'])) for im in imfinedet]  # shift objects only, equivalent to untile() but faster
+                imcoarsedet = imcoarsedet.objects( imcoarsedet.objects() + [o for im in imfinedet for o in im.objects()])  # union
             else:
                 imfinedet = iml.untile( im_multiscale[nc:] )
-            imcoarsedet = imcoarsedet.union(imfinedet) if imfinedet is not None else imcoarsedet
-            imlistdet.append(imcoarsedet.nms(conf, iou))
+                imcoarsedet = imcoarsedet.union(imfinedet) if imfinedet is not None else imcoarsedet
+            imlistdet.append(imcoarsedet.nms(conf, iou, cover=cover))
 
         return imlistdet[0] if len(imlistdet) == 1 else imlistdet
 
@@ -306,7 +324,7 @@ class VideoTracker(ObjectDetector):
 class MultiscaleVideoTracker(MultiscaleObjectDetector):
     """MultiscaleVideoTracker() class"""
 
-    def __init__(self, minconf=0.001, miniou=0.6, maxhistory=5, smoothing=None, objects=None, trackconf=0.05, verbose=False, gpu=None, batchsize=1, weightfile=None, overlapfrac=2):
+    def __init__(self, minconf=0.001, miniou=0.6, maxhistory=5, smoothing=None, objects=None, trackconf=0.05, verbose=False, gpu=None, batchsize=1, weightfile=None, overlapfrac=2, detbatchsize=None, gate=0):
         super().__init__(gpu=gpu, batchsize=batchsize, weightfile=weightfile)
         self._minconf = minconf
         self._miniou = miniou
@@ -317,18 +335,20 @@ class MultiscaleVideoTracker(MultiscaleObjectDetector):
         self._verbose = verbose
         self._maxarea = 1.0
         self._overlapfrac = overlapfrac
+        self._detbatchsize = detbatchsize if detbatchsize is not None else self.batchsize()
+        self._gate = gate
         
     def __call__(self, vi, stride=1):
         """Yield vipy.video.Scene(), an incremental tracked result for each frame"""
         assert isinstance(vi, vipy.video.Video), "Invalid input"
 
         (det, n) = (super().__call__, self._mindim)
-        for (k, vb) in enumerate(vi.stream().batch(self.batchsize())):
+        for (k, vb) in enumerate(vi.stream().batch(self._detbatchsize)):
             framelist = vb.framelist()
             for (j, im) in zip(range(0, len(framelist), stride), tolist(det(framelist[::stride], self._minconf, self._miniou, self._maxarea, objects=self._objects, overlapfrac=self._overlapfrac))):
                 for i in range(j, j+stride):                    
                     if i < len(framelist):
-                        yield vi.assign(k*self.batchsize()+i, im.objects(), minconf=self._trackconf, maxhistory=self._maxhistory) if (i%stride == 0) else vi
+                        yield (vi.assign(k*self._detbatchsize+i, im.objects(), minconf=self._trackconf, maxhistory=self._maxhistory, gate=self._gate) if (i == j) else vi)
 
     def stream(self, vi):
         return self.__call__(vi)
